# libguestfs Python bindings
# Copyright (C) 2010 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import unittest
import guestfs


class Test070OptArgs(unittest.TestCase):
    def setUp(self):
        self.g = guestfs.GuestFS(python_return_dict=True)

    def test_no_optargs(self):
        self.g.add_drive("/dev/null")

    def test_one_optarg(self):
        self.g.add_drive("/dev/null", readonly=True)

    def test_two_optargs(self):
        self.g.add_drive("/dev/null", iface="virtio", format="raw")

    def tearDown(self):
        self.g.close()
